using DBHelper;
using DBHelper.Redis;
using LogHelper;
using Newtonsoft.Json;
using ServiceCenter;
using ServiceCenter.Virtual_PLC;
using SqlSugar;
using System.Text;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol;
using WCS.Entity.Protocol.BCR;
using WCS.Entity.Protocol.Station;
using WCS.Service.Systems;

namespace WCS.Service
{
    /// <summary>
    /// 
    /// </summary>
    public class Worker : BackgroundService
    {
        /// <summary>
        /// ¼
        /// </summary>
        private readonly ILogger<Worker> _logger;

        /// <summary>
        /// 캯
        /// </summary>
        /// <param name="logger">¼</param>
        public Worker(ILogger<Worker> logger)
        {
            _logger = logger;
        }

        public static readonly string WcsDlog = "WCSDlog";
        public static readonly string Wcsdb = "WCSDB";

        /// <summary>
        ///  ִ
        /// </summary>
        /// <param name="stoppingToken">ֹͣ</param>
        /// <returns></returns>
        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            if (stoppingToken.IsCancellationRequested)
                return;

            #region ־

            var logConfigText = RedisHub.Default.Check("LogConfigText") ?? throw new Exception("Redislog4net");
            var logConfig = JsonConvert.DeserializeObject<LogConfig>(logConfigText);
            LogHub.SetConfigInfo(logConfig!);

            #endregion ־

            _logger.LogInformation("WCSʼ");
            Configs.ProtocolProxyBaseType = typeof(ProtocolProxy);
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            Configs.StringEncoding = Encoding.UTF8;
            var warehouseName = RedisHub.Default.Check("WarehouseName") ?? throw new Exception("Redisòֿ");
            if (string.IsNullOrEmpty(warehouseName)) throw new Exception("Redisòֿ");
            ServiceHub.SetWarehouseName(warehouseName);

            #region ʼݿ

            var dbConnectionStrings = RedisHub.Default.Check("DbConnectionStrings") ?? throw new Exception("Redisݿ");
            ServiceHub.DbConnectionStrings = JsonConvert.DeserializeObject<List<DataBaseConnectionString>>(dbConnectionStrings);
            if (ServiceHub.DbConnectionStrings != null)
            {
                if (ServiceHub.DbConnectionStrings.All(v => v.Key != Wcsdb)) throw new Exception("DbConnectionStringsWCSݿַ");
                if (ServiceHub.DbConnectionStrings.All(v => v.Key == Wcsdb && !v.IsDefault)) throw new Exception("DbConnectionStringsWCSݿΪĬݿ");
                if (ServiceHub.DbConnectionStrings.All(v => v.Key != WcsDlog)) throw new Exception("DbConnectionStringsWCS־ݿַ");
            }

            foreach (var connectionString in ServiceHub.DbConnectionStrings!)
            {
                Db.CreateContext(new ConnectionConfig()
                {
                    ConnectionString = connectionString.ConnectionString,
                    DbType = connectionString.DbType
                }, connectionString.Key);
                if (connectionString.IsDefault) Db.SetDefaultDbContextType(connectionString.Key);

                switch (connectionString.Key)
                {
                    case "WCSDB"://WCSݿ
                        Db.Do(db =>
                        {
                            db.Default.CodeFirst.InitTables(typeof(WCS_PLC));
                            db.Default.CodeFirst.InitTables(typeof(WCS_DATABLOCK));
                            db.Default.CodeFirst.InitTables(typeof(WCS_DEVICEHdr));
                            db.Default.CodeFirst.InitTables(typeof(WCS_PathHdr));
                            db.Default.CodeFirst.InitTables(typeof(WCS_PathDtl));
                            db.Default.CodeFirst.InitTables(typeof(WCS_ROUTE));
                            db.Default.CodeFirst.InitTables(typeof(WCS_TASK));
                            db.Default.CodeFirst.InitTables(typeof(WCS_TASK_DTL));
                            db.Default.CodeFirst.InitTables(typeof(WCS_TASK_OLD));
                            db.Default.CodeFirst.InitTables(typeof(WCS_PlcData));
                            db.Default.CodeFirst.InitTables(typeof(WCS_AGVTask));
                            db.Default.CodeFirst.InitTables(typeof(WCS_DEVICEPROTOCOL));
                            db.Default.CodeFirst.InitTables(typeof(WCS_GROUPMEMBER));

                            //db.Default.CodeFirst.InitTables(typeof(WCS_BCR80));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_RGV520));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_RGV521));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_RGV523));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_SRM520));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_SRM521));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_SRM537));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_Station520));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_Station521));
                            //db.Default.CodeFirst.InitTables(typeof(WCS_Station523));
                        });
                        break;

                    case "WCSDlog"://WCS־ݿ
                        Db.Do(db =>
                        {
                            db.Context(WcsDlog).DbMaintenance.CreateDatabase();
                            db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_BCR80));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV520));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV521));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_RGV523));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_SRM520));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_SRM521));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_SRM537));
                            db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_Station520));
                            db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_Station521));
                            db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_Station523));
                            db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS_Station91));
                            //db.Context(WcsDlog).CodeFirst.InitTables(typeof(WCS.Entity.PlcRawData));
                        });
                        break;

                    default: //
                        break;
                };
            };

            #endregion ʼݿ

            #region PLC

            var isOpenVirtualPlc = RedisHub.Default.Check("isOpenVirtualPLC") ?? throw new Exception("RedsiǷPLC");
            if (isOpenVirtualPlc == "1")
            {
                var plcDataConnectionString = RedisHub.Default.Check("plcDataConnectionString") ?? throw new Exception("RedsiPLCʹõRedisַ");

                //нṹҪĽṹ
                var list = new List<PLCData>();
                Db.Do(db =>
                {
                    var dataBlocks = db.Default.Queryable<WCS_DATABLOCK>().Includes(v => v.PLC).ToList();
                    list.AddRange(dataBlocks.Select(dataBlock => new PLCData()
                    {
                        IP = dataBlock.PLC.IP,
                        DB = dataBlock.NO,
                        Length = dataBlock.LENGTH,
                        DataLength = dataBlock.DATALENGTH,
                    }));
                });
                PlcData.Init(plcDataConnectionString).InitPlcData(list);

                ServiceHub.AddSystemMode(SystemMode.plc);
            }

            #endregion PLC

            //־¼s
            Configs.PublishEvent += () =>
            {
                //WMS.UploadDevInfo();
                //ProtocolProxy.Do();
            };
            //쳣
            Configs.UploadException = (d, s) =>
            {
                //if (s == "ӿڵ") return;
                //if (ProtocolProxy.AllDatas.ContainsKey(d))
                //{
                //    ProtocolProxy.AllDatas[d].Info = s;
                //    //ProtocolProxy.AllDatas[d].Frame = LogicHandler.Frame;
                //}
                //WMS.TaskException(d, s);
            };
            //PLC
            Configs.PLCAccessorCreater = new PLCAccessors.PLCAccessorsCreater();
            try
            {
                Db.Do(db =>
                {
                    //ȡDBдЭ
                    var dbProtocols = db.Default.Queryable<WCS_DEVICEPROTOCOL>().Includes(v => v.DB, p => p.PLC).ToList();
                    foreach (var dbProtocol in dbProtocols)
                    {
                        if (dbProtocol.DEVICECODE.Contains("SRM"))
                        {
                        }
                        else if (dbProtocol.DEVICECODE.Contains("RGV"))
                        {
                        }
                        else if (dbProtocol.DEVICECODE.Contains("BCR"))
                        {
                        }
                        else
                        {
                            if (dbProtocol.DB.CODE.Contains("520"))
                            {
                                Add<IStation520>(dbProtocol.DEVICECODE, dbProtocol.POSITION, (ushort)dbProtocol.DB.NO, dbProtocol.DB.PLC.IP);
                            }
                            else if (dbProtocol.DB.CODE.Contains("521"))
                            {
                                Add<IStation521>(dbProtocol.DEVICECODE, dbProtocol.POSITION, (ushort)dbProtocol.DB.NO, dbProtocol.DB.PLC.IP);
                            }
                            else if (dbProtocol.DB.CODE.Contains("523"))
                            {
                                Add<IStation522>(dbProtocol.DEVICECODE, dbProtocol.POSITION, (ushort)dbProtocol.DB.NO, dbProtocol.DB.PLC.IP);
                            }
                            else
                            {
                                continue;
                            }
                        }
                    }
                });

                #region е

                World.StartAll();

                #endregion е

                _logger.LogInformation("WCSɹ");

                #region ݲɼ

                while (true)
                {
                    Ltc.GetSystem<DataCollectionSysyem>().Invoke(true);
                }

                #endregion ݲɼ
            }
            catch (Exception ex)
            {
                _logger.LogError("WCSʧ{0}", ex.Message);
            }
        }

        /// <summary>
        ///  Э
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="code"> </param>
        /// <param name="position"> </param>
        /// <param name="db"></param>
        /// <param name="ip"></param>
        public static void Add<T>(string code, int position, ushort db, string ip)
        {
            var info = new ProtocolInfo
            {
                Position = position,
                DBInfo = new DBInfo
                {
                    No = db,
                    PLCInfo = new PLCInfo
                    {
                        IP = ip,
                        Port = 102,
                        Rack = 0,
                        Slot = 1,
                        Type = Core.PLCType.Siemens
                    }
                }
            };
            Protocols<T>.Add(code, info);
        }
    }
}
